#include <stdio.h>
#include <stdlib.h>
#include <conio.h>

float s[360],c[360];

void MakeLookUp();
void RotateObject(D3Object_ptr Object, int angleX, int angleY, int angleZ);
void RenderObject(D3Object_ptr Object,int xm, int ym);
void RotateVertexNormals(D3Object_ptr Object, int angleX, int angleY, int angleZ);
void LightenObject(D3Object_ptr Object, D3Vertex_ptr SpotLight);

// Make a lookuptable to save time calculating cos and sin
void MakeLookUp()
{	for (int count=0;count<=359;count++)
	{	s[count]=sin(count*0.017453292);
		c[count]=cos(count*0.017453292);
	}
}

// Standard rotations
void RotateObject(D3Object_ptr Object, int angleX, int angleY, int angleZ)
{	float temp_x,temp_y,temp_z;

	for (int count=0;count<Object->NumVertices;count++)
	{	temp_x	= c[angleX] * Object->Vertices_local[count].x - s[angleX] * Object->Vertices_local[count].y;
		temp_y	= s[angleX] * Object->Vertices_local[count].x + c[angleX] * Object->Vertices_local[count].y;
		
		Object->Vertices_world[count].x = c[angleY] * temp_x - s[angleY] * Object->Vertices_local[count].z;
		temp_z							= s[angleY] * temp_x + c[angleY] * Object->Vertices_local[count].z;

		Object->Vertices_world[count].y = c[angleZ] * temp_y - s[angleZ] * temp_z;
		Object->Vertices_world[count].z = c[angleZ] * temp_z + s[angleZ] * temp_y;
	}
}

// I use Windowcoords' so we'll need to scale the vertices ourselves !
// Just divide them by there distance, but be carefull not to divide by zero
void RenderObject(D3Object_ptr Object,int xm,int ym)
{	float scale;
	int   dist=500;	

	for (int count=0;count<Object->NumVertices;count++)
	{	scale = dist/((float)Object->Vertices_world[count].z+dist);
		Object->Vertices_screen[count].x   = xm + Object->Vertices_world[count].x * scale ;
		Object->Vertices_screen[count].y   = ym + Object->Vertices_world[count].y * scale ;
		Object->Vertices_screen[count].oow = 1.0 / (dist - Object->Vertices_world[count].z);
	}
	// This function doesn't do clipping and we use a depthbuffer so every triangle can be drawing
	for (count=0;count<Object->NumFaces;count++)
	{	grDrawTriangle(	&(Object->Vertices_screen[Object->Faces[count].a]),
						&(Object->Vertices_screen[Object->Faces[count].b]),
						&(Object->Vertices_screen[Object->Faces[count].c]) );
	}
}







void RotateVertexNormals(D3Object_ptr Object, int angleX, int angleY, int angleZ)
{	float temp_x,temp_y,temp_z;
	
	// Instead of rotating all the vertex normals I only rotate the ligthsource in the opposite direction, relative to the object...
	// It isn't really a clean way of doing it but it sure saves some processing time !
	angleX = 359 - angleX;
	angleY = 359 - angleY;
	angleZ = 359 - angleZ;

	temp_x	= c[angleX] * Object->SpotlightStat.x - s[angleX] * Object->SpotlightStat.y;
	temp_y	= s[angleX] * Object->SpotlightStat.x + c[angleX] * Object->SpotlightStat.y;
		
	Object->Spotlight.x = c[angleY] * temp_x - s[angleY] * Object->SpotlightStat.z;
	temp_z				= s[angleY] * temp_x + c[angleY] * Object->SpotlightStat.z;

	Object->Spotlight.y = c[angleZ] * temp_y - s[angleZ] * temp_z;
	Object->Spotlight.z = c[angleZ] * temp_z + s[angleZ] * temp_y;
}



void LightenObject(D3Object_ptr Object)
{	// function calculates the amount of light at every vertex
	// Just calculat the angle between every vertexnormal, which we have precalculated, and the ligthsource.
	// This is all standard  highschool math.
	float x,y,z;
	float angle;
	for (int count=0;count<Object->NumVertices;count++)
	{	x = Object->Vertices_normalStat[count].x;
		y = Object->Vertices_normalStat[count].y;
		z = Object->Vertices_normalStat[count].z;
		
		// 866  is the distance to the lightsource ( SQRT((500^2)+(500^2)+(500^2)) )
		// officially we would also divide "angle" by the length of the Vertexnormal but we normalised it to 1 so there is no need for that
		// You don't want to calculate a sqrt for every Vertex !
		// Ofcource we should eliminate the cos but I don't have any good idea's for doing this
		// Suggestions ?
		angle = 1-fabs(cos(( (x * Object->Spotlight.x) + (y * Object->Spotlight.y) + (z * Object->Spotlight.z) )/866));
		
		// Now scale every colorcomponent by "angle" as this is a value in the range: (1-abs(-1..1)) -> (0..1)
		Object->Vertices_screen[count].r = Object->Vertices_local[count].r * angle;
		Object->Vertices_screen[count].b = Object->Vertices_local[count].b * angle;
		Object->Vertices_screen[count].g = Object->Vertices_local[count].g * angle;
	}
}