#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>

#define ReadTil(string) while (strcmp(line, string)) fscanf(fp, "%s",line);

bool LoadAscFile(char *filename, D3Object_ptr Object, float scale);
void PrecomputeVertexNormals(D3Object_ptr Object);
bool SetupGlide();
void ShutdownGlide();
void MakeLookUp();

float s[361],c[361];
float negs[361],negc[361];



bool SetupGlide()
{	grGlideInit();
	
	long n;	

	if (! grGet(GR_NUM_BOARDS, sizeof(n), &n))
	{	printf("Error: Sorry, no 3DFX Graphics Accelerator detected...");
		exit(false);
	}
	
	grSstSelect(0);
	
	grSstWinOpen(NULL,	GR_RESOLUTION_800x600, 
						GR_REFRESH_60Hz, 
						GR_COLORFORMAT_RGBA, 
						GR_ORIGIN_UPPER_LEFT, 2, 0);

	grColorCombine(	GR_COMBINE_FUNCTION_LOCAL, 
					GR_COMBINE_FACTOR_NONE,
					GR_COMBINE_LOCAL_ITERATED, 
					GR_COMBINE_OTHER_NONE, FXFALSE);
	
	SetVertexLayout();

	grClipWindow(LeftClip,UpperClip,RightClip,LowerClip);

	grEnable(GR_SHAMELESS_PLUG);

	grDepthBufferMode( GR_DEPTHBUFFER_WBUFFER );
	grDepthBufferFunction( GR_CMP_GREATER );
	grDepthMask( FXTRUE );
	
	MakeLookUp();

	return true;
}

void ShutdownGlide()
{	grGlideShutdown();
}

void MakeLookUp()
{	for (int count=0;count<360;count++)
	{	s[count]=sin(count*0.017453292);
		c[count]=cos(count*0.017453292);
	}
	s[360]=s[0];
	c[360]=c[0];

	for (count=0;count<360;count++)
	{	negs[count]=sin(-count*0.017453292);
		negc[count]=cos(-count*0.017453292);
	}
	negs[360]=s[0];
	negc[360]=c[0];
}

bool LoadAscFile(char *filename, D3Object_ptr Object, float scale)
{	FILE  *fp;
	char  line[80];
	char  trash[20];
	D3Vertex_ptr	D3VertexList = NULL;
	Face_ptr		Faces		 = NULL;



	float x,y,z;

	if ((fp=fopen(filename,"r" ))==NULL)					
	{	printf("Error opening file: Can't find file\n");
		return(false);
	}
	
	fgets(line,80,fp);fgets(line,80,fp);
	fgets(line,80,fp);fgets(line,80,fp);

	sscanf(line,"Tri-mesh, Vertices: %d Faces: %d", &Object->NumVertices, &Object->NumFaces);
	
	D3VertexList = (D3Vertex_ptr) calloc(Object->NumVertices, sizeof(D3Vertex) );
	Faces		 = (Face_ptr)     calloc(Object->NumFaces, sizeof(Face));

	
	fgets(line,80,fp);								
	for (int count=0; count<Object->NumVertices; count++)
	{	ReadTil("Vertex");
		fgets(line,80,fp);
		
		sscanf(line,"%s X: %f Y: %f Z: %f", &trash, &x, &y, &z);
		D3VertexList[count].x=x * scale;
		D3VertexList[count].y=z * scale;
		D3VertexList[count].z=y * scale;
	}
	fgets(line,80,fp);
	for (count=0; count<Object->NumFaces; count++)
	{	ReadTil("Face");
		fgets(line,80,fp);
		sscanf(line,"%s A:%d B:%d C:%d",&trash, &Faces[count].a, &Faces[count].b, &Faces[count].c);
	}

	Object->Vertices_local      = D3VertexList;
	Object->Faces				= Faces;
	Object->Vertices_world      = (D3Vertex_ptr) calloc(Object->NumVertices, sizeof(D3Vertex) );
	Object->Vertices_screen     = (grVertex_ptr) calloc(Object->NumVertices, sizeof(grVertex) );
	Object->Vertices_normalStat = (Normal_ptr)   calloc(Object->NumVertices, sizeof(Normal)   );
	
	PrecomputeVertexNormals(Object);

	fclose(fp);
	return(true);
}




void PrecomputeVertexNormals(D3Object_ptr Object)
{	Normal a,b;
	int    deler;
	float  x,y,z,length;	
	
	for (int count=0;count<Object->NumVertices; count++)
	{	deler = 0;
		for (int count2=0;count2<Object->NumFaces; count2++)
		{	if(	Object->Faces[count2].a==count || Object->Faces[count2].b==count || Object->Faces[count2].c==count   )
			{	deler++;
				a.x = Object->Vertices_local[Object->Faces[count2].c].x - Object->Vertices_local[Object->Faces[count2].a].x;
				a.y = Object->Vertices_local[Object->Faces[count2].c].y - Object->Vertices_local[Object->Faces[count2].a].y;
				a.z = Object->Vertices_local[Object->Faces[count2].c].z - Object->Vertices_local[Object->Faces[count2].a].z;
				b.x = Object->Vertices_local[Object->Faces[count2].b].x - Object->Vertices_local[Object->Faces[count2].a].x;
				b.y = Object->Vertices_local[Object->Faces[count2].b].y - Object->Vertices_local[Object->Faces[count2].a].y;
				b.z = Object->Vertices_local[Object->Faces[count2].b].z - Object->Vertices_local[Object->Faces[count2].a].z;	
				
				x = x + ((b.y*a.z)-(b.z*a.y));	
				y = y + ((b.z*a.x)-(b.x*a.z));
				z = z + ((b.x*a.y)-(b.y*a.x));
			}	
		}
		if (deler != 0)
		{	Object->Vertices_normalStat[count].x = x / deler; 
			Object->Vertices_normalStat[count].y = y / deler; 
			Object->Vertices_normalStat[count].z = z / deler; 
		} 
		length = sqrt((Object->Vertices_normalStat[count].x*Object->Vertices_normalStat[count].x) + 
					  (Object->Vertices_normalStat[count].y*Object->Vertices_normalStat[count].y) + 
					  (Object->Vertices_normalStat[count].z*Object->Vertices_normalStat[count].z) );
		
		Object->Vertices_normalStat[count].x=-Object->Vertices_normalStat[count].x / length;
		Object->Vertices_normalStat[count].y=-Object->Vertices_normalStat[count].y / length;
		Object->Vertices_normalStat[count].z=-Object->Vertices_normalStat[count].z / length;
		
		x=0;y=0;z=0;
	}
}


