
// By Andreas Ingo 1998

// This program is running on top of glide3 demonstrating some of
// glide 3's new functions such as ClipCoordinates, Triangle setup
// using strips and fans. Demo is using texture mapping and w buffering
// too just to show you how it all works.

// This source have been successfully compiled using lccwin32 compiler
// using these command lines:
//
// lcc g3test.c
// lcclnk g3test.obj glide3.obj
//
// You need to have followed my glide3 guide at http://welcome.to/3dfxPS
// to be able to compile this program using lccwin32.
//


#include <glide3.h>     // Must include glide3.h instead of glide.h too
                        // make it work with other compilers then MSVC++ 

#include <stdio.h>
#include <math.h>
#include <conio.h>

typedef struct 
{
  float x,y;
  float z;
  float r,g,b;
  float u,v;
} MyVertex;

// Some stupid texture structure taken from FxEngine...
typedef struct fxeTexture
{
  void *Prev;
  void *Next;
  GrTexInfo TexInfo;
  
  int FilteringMethod;
  FxU32 VideoMemoryAddressTMU0;
  FxU32 VideoMemoryAddressTMU1;
  int DownloadedToVRAM;
  int UsedInFrame;
  void *TextureData;
} fxeTexture;

int main(void)
{
  MyVertex Vertex[4];
  FxI32 NumTexelFx;
  FxU32 WMaxValuePtr[2];
  float value, sign;
  MyVertex *VertexPts[5];
	Gu3dfInfo TexInfo;
  fxeTexture *Texture;

  // MUST call this function before using any glide function:
  LoadProcAddresses();


  grGlideInit();
  grSstSelect(0);


  if (grSstWinOpen((FxU32)NULL,GR_RESOLUTION_640x480,GR_REFRESH_60Hz,GR_COLORFORMAT_ARGB,
    GR_ORIGIN_LOWER_LEFT,2,1) == 0)
  {
    grGlideShutdown();
    return FXFALSE;
  }

  // Retrieve the number of TexelFx chips:

  grGet(GR_NUM_TMU,sizeof(NumTexelFx),&NumTexelFx);

  grDepthBufferMode(GR_DEPTHBUFFER_WBUFFER);
  grDepthMask(FXTRUE);
  grDepthBufferFunction(GR_CMP_LESS);

  grColorCombine(GR_COMBINE_FUNCTION_SCALE_OTHER,GR_COMBINE_FACTOR_LOCAL,
    GR_COMBINE_LOCAL_ITERATED,GR_COMBINE_OTHER_TEXTURE,FXFALSE);

  grSstOrigin(GR_ORIGIN_LOWER_LEFT);

  // Setup the vertex layout:
  grVertexLayout(GR_PARAM_XY,0,GR_PARAM_ENABLE);
  grVertexLayout(GR_PARAM_W,8,GR_PARAM_ENABLE);
  grVertexLayout(GR_PARAM_RGB,12,GR_PARAM_ENABLE);
  grVertexLayout(GR_PARAM_ST0,24,GR_PARAM_ENABLE);

  // Set coordinate space to clip coordinates
  grCoordinateSpace(GR_CLIP_COORDS);


  grViewport(0,0,(FxI32)640,(FxI32)480);

  grDepthRange(0,1);

    // Set up texture unit 0 for bilinear filtering
    grTexFilterMode(GR_TMU0,GR_TEXTUREFILTER_BILINEAR,
      GR_TEXTUREFILTER_BILINEAR);

    // Setup TMU0 for simple texture mapping
    grTexCombine(GR_TMU0,GR_COMBINE_FUNCTION_LOCAL,
      GR_COMBINE_FACTOR_NONE,
      GR_COMBINE_FUNCTION_LOCAL,
      GR_COMBINE_FACTOR_NONE,
      FXFALSE,FXFALSE);

    // enable mipmapping and mipmap blending:

  grTexClampMode(GR_TMU0,GR_TEXTURECLAMP_WRAP,GR_TEXTURECLAMP_WRAP);

	// get the texture info:
  gu3dfGetInfo("marble.3df",&TexInfo);

  // allocate memory for the mipmap:
  TexInfo.data = (void*)malloc(TexInfo.mem_required);

  // Load the mipmap from disk:
  gu3dfLoad("marble.3df",&TexInfo);


	Texture = (fxeTexture*)malloc(sizeof(fxeTexture));

  Texture->TexInfo.data = TexInfo.data;
	Texture->TexInfo.format = TexInfo.header.format;
  Texture->TexInfo.aspectRatioLog2 = TexInfo.header.aspect_ratio;
  Texture->TexInfo.smallLodLog2 = TexInfo.header.small_lod;
  Texture->TexInfo.largeLodLog2 = TexInfo.header.large_lod;


  grTexDownloadMipMap(GR_TMU0,grTexMinAddress(GR_TMU0),
  	GR_MIPMAPLEVELMASK_BOTH,&Texture->TexInfo);

  Texture->VideoMemoryAddressTMU0 = grTexMinAddress(GR_TMU0);

  grTexSource(GR_TMU0,Texture->VideoMemoryAddressTMU0,
    GR_MIPMAPLEVELMASK_BOTH,&Texture->TexInfo);



  grGet(GR_WDEPTH_MIN_MAX,8,(FxI32*)WMaxValuePtr);

  grConstantColorValue(255);

  value = 1;
  sign = 10;

  VertexPts[0] = Vertex;
  VertexPts[1] = &Vertex[1];
  VertexPts[2] = &Vertex[2];
  VertexPts[3] = &Vertex[3];
  VertexPts[4] = &Vertex[3];

  while (!kbhit())
    {
  grBufferClear(0,0,WMaxValuePtr[1]);

    if (value>=1000 || value <= 0)
      sign=-sign;
    value=value+sign;

    Vertex[0].x = -1000.f;
    Vertex[0].y = -1000.f;
    Vertex[0].z = 2000+value;
    Vertex[0].r = 1.f;
    Vertex[0].g = 0.f;
    Vertex[0].b = 0.f;
    Vertex[0].u = 0;
    Vertex[0].v = 1.0;

    Vertex[1].x = 1000.f;
    Vertex[1].y = 1000.f;
    Vertex[1].z = 2000+value;
    Vertex[1].r = 0.f;
    Vertex[1].g = 1.f;
    Vertex[1].b = 0.f;
    Vertex[1].u = 1.0;
    Vertex[1].v = 0;

    Vertex[2].x = 1000.f;
    Vertex[2].y = -1000.f;
    Vertex[2].z = 2000.f+value;
    Vertex[2].r = 0.f;
    Vertex[2].g = 0.f;
    Vertex[2].b = 1.f;
    Vertex[2].u = 1.0;
    Vertex[2].v = 1.0;

    grDrawVertexArray(GR_TRIANGLE_FAN,3,VertexPts);

    Vertex[3].x = -1000.f;
    Vertex[3].y = 1000.f;
    Vertex[3].z = 1000+value;
    Vertex[3].r = 1;
    Vertex[3].g = 1;
    Vertex[3].b = 1;
    Vertex[3].u = 0;
    Vertex[3].v = 0;

    Vertex[1].x = 1000.f;
    Vertex[1].y = 1000.f;
    Vertex[1].z = 1000+value;
    Vertex[1].r = 0.f;
    Vertex[1].g = 1.f;
    Vertex[1].b = 0.f;
    Vertex[1].u = 1.0;
    Vertex[1].v = 0;

    Vertex[2].x = 1000.f;
    Vertex[2].y = -1000.f;
    Vertex[2].z = 1000.f+value;
    Vertex[2].r = 0.f;
    Vertex[2].g = 0.f;
    Vertex[2].b = 1.f;
    Vertex[2].u = 1.0;
    Vertex[2].v = 1.0;


    grDrawVertexArray(GR_TRIANGLE_FAN,4,&VertexPts[1]);

    grBufferSwap(1);

    }

    getch();
    grGlideShutdown();
}

